@@ -49,6 +49,7 @@
 #include <ctype.h>
 #include <libgen.h>
 
+#include "cfgparser/olsrd_conf_checksum.h"
 #include "ipcalc.h"
 #include "builddata.h"
 #include "neighbor_table.h"
@@ -210,10 +211,12 @@ bool isCommand(const char *str, unsigned long long siw) {
 }
 
 const char * determine_mime_type(unsigned int send_what) {
-  return (send_what & SIW_OLSRD_CONF) ? "text/plain; charset=utf-8" : "application/json; charset=utf-8";
+  return (send_what & SIW_OLSRD_CONF) ? "text/plain; charset=utf-8" : "application/vnd.api+json";
 }
 
 void output_start(struct autobuf *abuf) {
+  char *str;
+
   /* global variables for tracking when to put a comma in for JSON */
   abuf_json_reset_entry_number_and_depth(&json_session, pretty);
   abuf_json_mark_output(&json_session, true, abuf);
@@ -221,6 +224,10 @@ void output_start(struct autobuf *abuf) {
   abuf_json_int(&json_session, abuf, "pid", getpid());
   abuf_json_int(&json_session, abuf, "systemTime", time(NULL));
   abuf_json_int(&json_session, abuf, "timeSinceStartup", now_times);
+
+  olsrd_config_checksum_get(NULL, &str);
+  abuf_json_string(&json_session, abuf, "configurationChecksum", str);
+
   if (*uuid) {
     abuf_json_string(&json_session, abuf, "uuid", uuid);
   }
@@ -1119,8 +1126,26 @@ void ipc_print_twohop(struct autobuf *abuf) {
 }
 
 void ipc_print_config(struct autobuf *abuf) {
+  char *str;
+
   abuf_json_mark_object(&json_session, true, false, abuf, "config");
 
+  olsrd_config_checksum_get(NULL, &str);
+  abuf_json_string(&json_session, abuf, "configurationChecksum", str);
+
+  {
+    size_t i = 0;
+    int argc = 0;
+    char **argv = NULL;
+    get_argc_argv(&argc, &argv);
+
+    abuf_json_mark_object(&json_session, true, true, abuf, "cli");
+    for (i = 0; i < (size_t) argc; i++) {
+      abuf_json_string(&json_session, abuf, NULL, argv[i]);
+    }
+    abuf_json_mark_object(&json_session, false, true, abuf, NULL);
+  }
+
   abuf_json_string(&json_session, abuf, "configurationFile", olsr_cnf->configuration_file);
   abuf_json_int(&json_session, abuf, "olsrPort", olsr_cnf->olsrport);
   abuf_json_int(&json_session, abuf, "debugLevel", olsr_cnf->debug_level);
